#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
Created on Sun Feb  18 02:37:00 2024

@author: leahghartman

Description : A file containing functions capable of analyzing a full dataset of images.
"""
# import random needed packages that should already be installed
import numpy as np
from PIL import Image

# import from other modules in the package
from gaussbean.analysis import single

#########################
### START OF FUNCTIONS
#########################

def full_set(imglist, xmargins, ymargins, pixelsize=3.45, xpixel=0, ypixel=0):
    """ Returns a list of FWHM values (in microns) for both x- and y-axes as well as a list of images cropped by the analysis function for troubleshooting and/or 
    making movies/animations/GIFs.

        Parameters
        ----------
        imglist : array
            Array of images (this needs to be a set of SORTED image paths (so, 1.tiff, 2.tiff, etc.).
        xmargins : integer
            How many pixels on each side of the beam (in the x-direction; located at the centroid of the image) you'd like as a buffer for cropping.
        ymargins : integer
            How many pixels on each side of the beam (in the y-direction; located at the centroid of the image) you'd like as a buffer for cropping.
        pixelsize : integer
            This function returns FWHM values in microns, which depends on the pixel size of your camera. The default for the CU-PWFA lab is 3.45 microns.
        lineout : boolean
            The user can choose whether they want to use lineouts or projections for full dataset analysis by specifying this variable to be what they want.
    """
    # create empty lists for FWHM in x- and y-directions as well as an empty list for all of the cropped images
    xlist = []
    ylist = []
    croppedimgs = []

    if xpixel == 0 & ypixel == 0:
        # for loop that cycles through all of the images and finds the FWHM along each axis (using PROJECTIONS)
        for i in imglist:
            # find the FWHM in both transverse dimensions as well as the cropped images used for processing
            xFWHM, yFWHM, croppedimg = single.single_image_proj(xmargins, ymargins, imgar=np.array(Image.open(i)))

            # append everything to their respective empty lists
            croppedimgs.append(croppedimg)
            xlist.append(xFWHM * pixelsize);
            ylist.append(yFWHM * pixelsize);

    else:
         # for loop that cycles through all of the images and finds the FWHM along each axis (using LINEOUTS)
        for i in imglist:
            # find the FWHM in both transverse dimensions as well as the cropped images used for processing
            xFWHM, yFWHM, croppedimg = single.single_image_line(xmargins, ymargins, xpixel=xpixel, ypixel=ypixel, imgar=np.array(Image.open(i)));

            # append everything to their respective empty lists
            croppedimgs.append(croppedimg)
            xlist.append(xFWHM * pixelsize);
            ylist.append(yFWHM * pixelsize);

    # return everything we want
    return(xlist, ylist, croppedimgs)


#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
Created on Fri Jan  26 01:00:00 2024

@author: leahghartman

Description : A file for calculations/functions used in both single-image and full-dataset analysis.
"""
# import random needed packages that should already be installed
import numpy as np
import matplotlib.pyplot as plt
from PIL import Image
from mpl_toolkits.axes_grid1 import make_axes_locatable
from scipy.signal import peak_widths, find_peaks

#########################
### START OF FUNCTIONS
#########################

def check_array(imgpath, imgar):
    """ Returns the image path or the array of the image based on what the user has input into the function that's calling check_array(). This function shouldn't be
    called by the user at any point.

        Parameters
        ----------
        imgpath : string
            The path to the image that the user wants to run through the median filter.
        imgar : array
            Array of the image if the user wants to input an array into the function rather than just an image path.
    """
    # if the length of the image array is empty (the user didn't want to use an array), we use the image path instead
    if len(imgar) == 0:
        return(np.array(Image.open(imgpath).convert("L")))
    # if the length of the image array isn't zero, the user wants to use an array instead of the image path, so we return the array that was input
    else:
        return(imgar)

########################################################

def find_FWHM(imgdata):
    """ Returns the Full-Width at Half-Maximum of a set of data. This function uses the most prominent peak to find the FWHM.

        Parameters
        ----------
        imgdata : array
            Data corresponding to a singular axis or a set of data that the user wants to find the FWHM of using the most prominent peak in the data.
    """
    # find the most prominent peak
    peakmax = np.max(imgdata)

    # use the maximum value (coresponding to the most prominent peak) to find the peak of the curve to find the FWHM of
    peaks, _ = find_peaks(imgdata, prominence=(peakmax/1.1, peakmax))

    # find the width (FWHM) of the most prominent peak
    results_half = peak_widths(imgdata, peaks, rel_height=0.5)

    # return the FWHM calculation
    return(results_half[0])

########################################################

def find_centroid(imgpath='', imgar=[]):
    """ Returns x- and y-coordinate of the centroid based on the MAXIMUM INTENSITY of the image in each transverse dimension.

        Parameters
        ----------
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Array of the image if the user wants to input an array into the function rather than just an image path.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = check_array(imgpath, imgar)

    # find the x- and y-coordinates of the centroid by doing a projection of the entire image and finding the maximum value in the array for each dimension
    centx, centy = np.argmax(arrayimg.sum(axis=0)), np.argmax(arrayimg.sum(axis=1))

    # return the centroid's coordinates
    return(centx, centy)

########################################################

def find_proj_x(imgpath='', imgar=[]):
    """ Returns the projection of an image along the x-axis.

        Parameters
        ----------
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Array of the image if the user wants to input an array into the function rather than just an image path.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = check_array(imgpath, imgar)

    # return the summation of EACH column (so, this is the projection along the x-axis)
    return(arrayimg.sum(axis=0))

########################################################

def find_proj_y(imgpath='', imgar=[]):
    """ Returns the projection of an image along the y-axis.

        Parameters
        ----------
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Array of the image if the user wants to input an array into the function rather than just an image path.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = check_array(imgpath, imgar)
    
    # return the summation of EACH row (so, this is the projection along the y-axis)
    return(arrayimg.sum(axis=1))

########################################################

def find_line_x(ypixel, toavg=0, imgpath='', imgar=[]):
    """ Returns the lineout of an image along the x-axis and averages multiple columns of pixels if the user wants.

        Parameters
        ----------
        ypixel : integer
            Specifies at what ROW the user wants to take the lineout along the image in pixels. If you're going to find an x lineout, you need to specify a y-pixel
            along which the lineout is going to lie.
        toavg (OPTIONAL) : integer
            Specifies the number of pixels on EACH SIDE of the original pixel lineout the user wants to create a projection with (so, center lineout, plus 
            two lineouts on either side if "toavg" is set equal to 2).
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Array of the image if the user wants to input an array into the function rather than just an image path.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = check_array(imgpath, imgar)

    # this isn't really needed functionality, but added a while loop that appends the all the lineouts the user wants to a list
    lineoutar = []
    i = -toavg;
    while i <= toavg:
        lineoutar.append(np.array(arrayimg[ypixel+i, :]))
        i += 1;

    # return the total projection of all of the lineouts
    return(np.array(lineoutar).sum(axis=0))

########################################################

def find_line_y(xpixel, toavg=0, imgpath='', imgar=[]):
    """ Returns the lineout of an image along the y-axis and averages multiple rows of pixels if the user wants.

        Parameters
        ----------
        xpixel : integer
            Specifies at what COLUMN the user wants to take the lineout along the image in pixels. If you're going to find a y lineout, you need to specify an 
            x-pixel along which the lineout is going to lie.
        toavg (OPTIONAL) : integer
            Specifies the number of pixels on EACH SIDE of the original pixel lineout the user wants to average with (so, center lineout, plus two lineouts on
            either side if "toavg" is set equal to 2).
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Array of the image if the user wants to input an array into the function rather than just an image path.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = check_array(imgpath, imgar)

    # this isn't really needed functionality, but added a while loop that appends the all the lineouts the user wants to a list
    lineoutar = []
    i = -toavg;
    while i <= toavg:
        lineoutar.append(np.array(arrayimg[:, xpixel+i]))
        i += 1;

    # return the total projection of all of the lineouts
    return(np.array(lineoutar).sum(axis=0))


#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
Created on Fri Jan  26 01:00:00 2024

@author: leahghartman

Description : A file for plotting utilities and functions that format plots.
"""
# import random needed packages that should already be installed
import cv2
import numpy as np
import matplotlib.pyplot as plt
from PIL import Image
from mpl_toolkits.axes_grid1 import make_axes_locatable

# import from other modules in the package
from gaussbean.utils import pre_utils, calc_utils

#########################
### START OF FUNCTIONS
#########################

def plot_beforeandafter(before, after, label='', clmap='plasma', fontsize=15):
    """ Returns a plot of an image before and after it has been run through a filter or altered in some way.

        Parameters
        ----------
        before : array
            The array associated with an image before it has been altered.
        after : array
            The array associated with an image after it has been altered.
        label (OPTIONAL) : string
            A string associated with WHAT is altering the image (could be "Median Filter" for example). Will be added to the title of the plot.
        clmap (OPTIONAL) : string
            The colormap that the user wants to use for the plots. This MUST be a colormap given by the matplotlib package.
        fontsize (OPTIONAL) : integer
            The fontsize used for the title of the plot. The axes labels are automatically formatted based on this number.
    """
    # format the figure so we have both images with before on the left and after on the right; add in some nice settings
    fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(8.5, 3))

    # set the title and axes labels for the image
    ax1.set_title('Image Before ' + label, fontsize=fontsize)
    ax2.set_title('Image After ' + label, fontsize=fontsize)
    ax1.set_ylabel('y pixels', fontsize=fontsize-3)
    ax1.set_xlabel('x pixels', fontsize=fontsize-3)
    
    ax2.set_ylabel('y pixels', fontsize=fontsize-3)
    ax2.set_xlabel('x pixels', fontsize=fontsize-3)

    # show the images
    ax1.imshow(before, cmap=clmap)
    ax2.imshow(after, cmap=clmap)

########################################################

def plot_median(mediansize, repeatamount=0, imgpath='', imgar=[], clmap='plasma', fontsize=15):
    """ Returns a plot of the image before and after it has been run through the median filter a specified number of times.

        Parameters
        ----------
        mediansize : integer
            The size of the median filter in pixels (generally want this to be small; from 2-10 pixels).
        repeatamount (OPTIONAL) : integer
            The times the user wants the filter to be run over the image.
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Rather than a file path, if one wants to use an array to represent the image instead, it's possible.
        clmap (OPTIONAL) : string
            The colormap that the user wants to use for the plots. This MUST be a colormap given by the matplotlib package.
        fontsize (OPTIONAL) : integer
            The fontsize used for the title of the plot. The axes labels are automatically formatted based on this number.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = calc_utils.check_array(imgpath, imgar)
    
    # get the image before the median filter is applied and after the filter is applied
    before = arrayimg
    after = pre_utils.thru_median(mediansize, repeatamount=repeatamount, imgar=arrayimg)

    # plot before and after
    plot_beforeandafter(before, after, label='Median Filter', clmap='plasma', fontsize=fontsize)

########################################################

def plot_lowpass(radius, imgpath='', imgar=[], clmap='plasma', fontsize=15):
    """ Returns a plot of the image before and after it has been run through a low-pass filter one time.

        Parameters
        ----------
        radius : integer
            The radius of the mask used for the low-pass filter in pixels.
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Rather than a file path, if one wants to use an array to represent the image instead, it's possible.
        clmap (OPTIONAL) : string
            The colormap that the user wants to use for the plots. This MUST be a colormap given by the matplotlib package.
        fontsize (OPTIONAL) : integer
            The fontsize used for the title of the plot. The axes labels are automatically formatted based on this number.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = calc_utils.check_array(imgpath, imgar)
    
    # get the image before the low-pass filter is applied and after the filter is applied
    before = arrayimg
    after = pre_utils.thru_lowpass(radius, imgar=arrayimg)

    # plot before and after
    plot_beforeandafter(before, after, label='Low-Pass Filter', clmap='plasma', fontsize=fontsize)

########################################################

def plot_medandlow(mediansize, radius, repeatamount=0, imgpath='', imgar=[], clmap='plasma', fontsize=15):
    """ Returns a plot of the original image, the image after ONLY a median filter has been applied, the image after ONLY a low-pass filter has been applied, and the image after BOTH a median filter and low-pass filter have been applied.

        Parameters
        ----------
        mediansize : integer
            The size of the median filter in pixels (generally want this to be small; from 2-10 pixels).
        radius : integer
            The radius of the mask used for the low-pass filter in pixels.
        repeatamount (OPTIONAL) : integer
            The times the user wants the MEDIAN filter to be run over the image.
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Rather than an image path, one can opt to use an array of the image instead.
        clmap (OPTIONAL) : string
            The colormap that the user wants to use for the plots. This MUST be a colormap given by the matplotlib package.
        fontsize (OPTIONAL) : integer
            The fontsize used for the title of the plot. The axes labels are automatically formatted based on this number.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = calc_utils.check_array(imgpath, imgar)
    
    # get the original image, the image after the median filter, the image after the low-pass filter, and the image after both filters
    original = arrayimg
    aftermed = pre_utils.thru_median(mediansize, repeatamount=repeatamount, imgar=arrayimg)
    afterlow = pre_utils.thru_lowpass(radius, imgar=arrayimg)
    afterboth = pre_utils.thru_lowpass(radius, imgar=aftermed)
    
    # create a figure with four subplots, show the images that are found above, and label each of them
    fig, ((ax1, ax2), (ax3, ax4)) = plt.subplots(2, 2)
    fig.suptitle('Original Image Compared to Combinations of Filters', fontsize=fontsize, y=1.01)
    ax1.imshow(original, cmap=clmap)
    ax1.set_title('Original Image', fontsize=fontsize-3)
    ax2.imshow(aftermed, cmap=clmap)
    ax2.set_title('After Median Filter', fontsize=fontsize-3)
    ax3.imshow(afterlow, cmap=clmap)
    ax3.set_title('After Low-Pass Filter', fontsize=fontsize-3)
    ax4.imshow(afterboth, cmap=clmap)
    ax4.set_title('After Both Filters', fontsize=fontsize-3)

    # label all x- and y-axes
    fig.text(0.5, 0.04, 'x pixels', ha='center', fontsize=fontsize-3)
    fig.text(0.04, 0.5, 'y pixels', va='center', rotation='vertical', fontsize=fontsize-3)

    # fix the axes ticks so they only show for outer plots
    for ax in fig.get_axes():
        ax.label_outer()

########################################################

def plot_cropped(xpoint, ypoint, xmargins, ymargins, imgpath='', imgar=[]):
    """ Returns a plot of the image before and after it's been cropped.

        Parameters
        ----------
        xpoint : integer
            x-coordinate of the point at which the crop will be occurring.
        ypoint : integer
            y-coordinate of the point at which the crop will be occurring.
        xmargins : integer
            A number (in pixels) of how far in the x-direction, on either side of the cropping point, the user wants the image to be cropped.
        ymargins : integer
            A number (in pixels) of how far in the y-direction, on either side of the cropping point, the user wants the image to be cropped.
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Array of the image if the user wants to input an array into the function rather than just an image path.
    """
    # get the array of the image before and after background subtraction
    before = calc_utils.check_array(imgpath, imgar)
    after = pre_utils.crop_image(xpoint, ypoint, xmargins, ymargins, imgar=before)
    
    # plot before and after doing background subtraction
    plot_beforeandafter(before, after, label='Cropping', clmap='plasma', fontsize=fontsize)

########################################################

def back_sub_plot(origpath='', backpath='', origimgar=[], backimgar=[], clmap='plasma', fontsize=15):
    """ Returns a plot of the image BEFORE and AFTER background subtraction.

        Parameters
        ----------
        origpath (OPTIONAL) : string
            The path of the original image to be used (the image before background subtraction).
        backpath (OPTIONAL) : string
            The path of the background image to be subtracted from the original image (this should be a blank image).
        origimgar (OPTIONAL) : array
            Rather than an image path, one can opt to use an array instead. This is the array of the original image to be used (before background subtraction).
        backimgar (OPTIONAL) : array
            Rather than an image path, one can opt to use an array instead. This is the array of the background image to be used (a blank image).
        clmap (OPTIONAL) : string
            The colormap that the user wants to use for the plots. This MUST be a colormap given by the matplotlib package.
        fontsize (OPTIONAL) : integer
            The fontsize used for the title of the plot. The axes labels are automatically formatted based on this number.
    """
    # get the array of the image before and after background subtraction
    before = calc_utils.check_array(origpath, origimgar)
    backimg = calc_utils.check_array(backpath, backimgar)
    after = pre_utils.back_subtract(origimgar=before, backimgar=backimg)
    
    # plot before and after doing background subtraction
    plot_beforeandafter(before, after, label='Background Subtraction', clmap=clmap, fontsize=fontsize)

########################################################

def plot_intensity_proj(imgpath='', imgar=[]):
    """ Returns a plot of the image with PROJECTIONS of the intensity on each axis.

        Parameters
        ----------
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Rather than a path, if the user wants to use an array for the image instead, it's possible.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = calc_utils.check_array(imgpath, imgar)

    # find the width and height of the image
    imheight, imwidth = np.shape(arrayimg)

    # create a numpy array so the axes will actually work on the right-hand-side graph (this isn't necessary on the top graph)
    positionsy = np.arange(1, imheight + 1, 1)
    positionsx = np.arange(1, imwidth + 1, 1)

    # customize the plots/plot as a whole (that is literally all that these lines of code do)
    fig, main_ax = plt.subplots(figsize=(7, 7))
    divider = make_axes_locatable(main_ax)
    top_ax = divider.append_axes("top", 1.05, pad=0.3, sharex=main_ax)
    right_ax = divider.append_axes("right", 1.05, pad=0.3, sharey=main_ax)

    # make the tick labels on the bottom sides of the top- and right-hand-side graphs disappear
    top_ax.xaxis.set_tick_params(labelbottom=False)
    right_ax.yaxis.set_tick_params(labelleft=False)
    right_ax.tick_params(labelrotation=-90)

    # give labels to all of the necessary axes and plots themselves (might have to play with the arangement of the right plot's title)
    main_ax.set_xlabel('x pixels', fontsize=13)
    main_ax.set_ylabel('y pixels', fontsize=13)
    top_ax.set_title('Intensity Profile (Projection) of Pixel Columns', fontsize=13)
    right_ax.set_title('Intensity Profile (Projection) of Pixel Rows', x=1.13, y=-0.05, rotation=-90, fontsize=13)

    # show the image as the main plot
    main_ax.imshow(arrayimg, extent=[0, imwidth, imheight, 0])

    # calculates the sum of the intensity values of all the pixels in every row and column
    cols = calc_utils.find_proj_x(imgar=arrayimg)
    rows = calc_utils.find_proj_y(imgar=arrayimg)

    # plots the right and top graphs with certain colors. If you want a different color, just change the 'color' input below
    v_prof, = right_ax.plot(rows, positionsy, color='black')
    h_prof, = top_ax.plot(cols, color='black')

    # show the entire figure
    plt.autoscale()
    plt.show()

########################################################

def plot_intensity_line(xpixel, ypixel, toavg=0, imgpath='', imgar=[]):
    """ Returns a plot of the image with LINEOUTS of the intensity on each axis. The location of the lineouts are specified by the user.

        Parameters
        ----------
        xpixel : integer
            Specifies the pixel at which the Y LINEOUT will be taken (for example, you want a y lineout, a straight line, you pick an x value, like x=0)
        ypixel : integer
            Specifies the pixel at which the X LINEOUT will be taken (for example, you want an x lineout, a straight line, you pick a y value, like y=0)
        toavg (OPTIONAL) : integer
            Specifies the number of pixels on EACH SIDE of the original pixel lineout the user wants to average with (so, center lineout, plus two lineouts on
            either side if "toavg" is set equal to 2).
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Rather than a path, if the user wants to use an array for the image instead, it's possible.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = calc_utils.check_array(imgpath, imgar)

    # find the width and height of the image
    imheight, imwidth = np.shape(arrayimg)

    # create a numpy array so the axes will actually work on the right-hand-side graph (this isn't necessary on the top graph)
    positionsy = np.arange(1, imheight + 1, 1)
    positionsx = np.arange(1, imwidth + 1, 1)

    # customize the plots/plot as a whole (that is literally all that these lines of code do)
    fig, main_ax = plt.subplots(figsize=(7, 7))
    divider = make_axes_locatable(main_ax)
    top_ax = divider.append_axes("top", 1.05, pad=0.3, sharex=main_ax)
    right_ax = divider.append_axes("right", 1.05, pad=0.3, sharey=main_ax)

    # make the tick labels on the bottom sides of the top- and right-hand-side graphs disappear
    top_ax.xaxis.set_tick_params(labelbottom=False)
    right_ax.yaxis.set_tick_params(labelleft=False)
    right_ax.tick_params(labelrotation=-90)

    # give labels to all of the necessary axes and plots themselves (might have to play with the arangement of the right plot's title)
    main_ax.set_xlabel('x pixels', fontsize=13)
    main_ax.set_ylabel('y pixels', fontsize=13)
    top_ax.set_title('Intensity Profile (Projection) of Pixel Columns', fontsize=13)
    right_ax.set_title('Intensity Profile (Projection) of Pixel Rows', x=1.13, y=-0.05, rotation=-90, fontsize=13)

    # show the image as the main plot
    main_ax.imshow(arrayimg, extent=[0, imwidth, imheight, 0])

    # calculates the sum of the intensity values of all the pixels in every row and column
    cols = calc_utils.find_line_x(ypixel, toavg=toavg, imgar=arrayimg)
    rows = calc_utils.find_line_y(xpixel, toavg=toavg, imgar=arrayimg)

    # plot the lineouts for nice visualization
    main_ax.plot(np.repeat(xpixel, len(arrayimg)), np.linspace(0, len(arrayimg), len(arrayimg)), color='y')
    main_ax.plot(np.linspace(0, len(arrayimg[0]), len(arrayimg[0])), np.repeat(ypixel, len(arrayimg[0])), color='y')
    main_ax.plot(xpixel, ypixel, '.', c='r')

    # plots the right and top graphs with certain colors. If you want a different color, just change the 'color' input below
    v_prof, = right_ax.plot(rows, positionsy, color='black')
    h_prof, = top_ax.plot(cols, color='black')

    # show the entire figure
    plt.autoscale()
    plt.show()



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
Created on Sun Feb  18 02:38:00 2024

@author: leahghartman

Description : A file containing functions capable of analyzing a single image in a dataset.
"""
# import random needed packages that should already be installed
import sys

# make sure we can get modules from the other directory
sys.path.append('../utils/')

# import from other modules in the package
from gaussbean.utils import pre_utils, calc_utils

#########################
### START OF FUNCTIONS
#########################

def single_image_proj(xmargins, ymargins, imgpath='', imgar=[]):
    """ Runs a data analysis algorithm on a single image. Returns the FWHM in both transverse dimensions (across the image) as well as the cropped image for
    diagnostic, GIF, or movie purposes.

        Parameters
        ----------
        xmargins : integer
            A number (in pixels) of how far in the x-direction, on either side of the cropping point, the user wants the image to be cropped.
        ymargins : integer
            A number (in pixels) of how far in the y-direction, on either side of the cropping point, the user wants the image to be cropped.
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Rather than a path to the image, one can also use the array of the image.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = calc_utils.check_array(imgpath, imgar)

    # crop out as many dead pixels as possible (as long as the feature is SOMEWHAT in the middle of the image, this should be fine)
    initialcrop = pre_utils.crop_image(1212, 1012, 1000, 988, imgar=arrayimg)

    # find the centroid of the image with as many of the large dead pixels cropped out as possible
    centx, centy = calc_utils.find_centroid(imgar=initialcrop)

    # crop the image for the final time. This is the image that will be used in the rest of the analysis process
    finalimg = pre_utils.crop_image(centx, centy, xmargins, ymargins, imgar=initialcrop)

    # find the centroid AGAIN, but more accurately, so we can get a really nicely cropped image
    centx2, centy2 = calc_utils.find_centroid(imgar=finalimg)
    
    # use the projection along the y-axis to find the FWHM value for the beam along the y-axis
    yFWHM = calc_utils.find_FWHM(calc_utils.find_proj_y(imgar=finalimg))[0]
    
    # use the projection along the x-axis to find the FWHM value for the beam along the x-axis
    xFWHM = calc_utils.find_FWHM(calc_utils.find_proj_x(imgar=finalimg))[0]

    # return the FWHM value for the beam along the x- and y- directions, as well as the final cropped image, which can be used for diagnostic purposes
    return(xFWHM, yFWHM, finalimg)

########################################################

def single_image_line(xmargins, ymargins, xpixel=0, ypixel=0, toavg=0, imgpath='', imgar=[]):
    """ Returns the image path or the array of the image based on what the user has input into the function that's calling check_array(). This function shouldn't be
    called by the user at any point.

        Parameters
        ----------
        xmargins : integer
            A number (in pixels) of how far in the x-direction, on either side of the cropping point, the user wants the image to be cropped.
        ymargins : integer
            A number (in pixels) of how far in the y-direction, on either side of the cropping point, the user wants the image to be cropped.
        xpixel (OPTIONAL) : integer
            Specifies at which COLUMN the user wants to take the lineout along the image in pixels.
        ypixel (OPTIONAL) : integer
            Specifies at which ROW the user wants to take the lineout along the image in pixels.
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Array of the image if the user wants to input an array into the function rather than just an image path.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = calc_utils.check_array(imgpath, imgar)

    # crop out as many dead pixels as possible (as long as the feature is SOMEWHAT in the middle of the image, this should be fine)
    initialcrop = pre_utils.crop_image(1212, 1012, 1000, 988, imgar=arrayimg)

    # find the centroid of the image with as many of the large dead pixels cropped out as possible
    centx, centy = calc_utils.find_centroid(imgar=initialcrop)

    # crop the image for the final time. This is the image that will be used in the rest of the analysis process
    finalimg = pre_utils.crop_image(centx, centy, xmargins, ymargins, imgar=initialcrop)

    # find the centroid AGAIN, but more accurately, so we can get a really nicely cropped image
    centx2, centy2 = calc_utils.find_centroid(imgar=finalimg)

    # if statement to see if the user wants to use their own selected lineouts, or if they want to use the centroid
    if xpixel == 0 & ypixel == 0:
        xpixel = centx2
        ypixel = centy2
    
    # use the projection along the y-axis to find the FWHM value for the beam along the y-axis
    yFWHM = calc_utils.find_FWHM(calc_utils.find_line_y(xpixel, toavg=toavg, imgar=finalimg))[0]
    
    # use the projection along the x-axis to find the FWHM value for the beam along the x-axis
    xFWHM = calc_utils.find_FWHM(calc_utils.find_line_x(ypixel, toavg=toavg, imgar=finalimg))[0]

    # return the FWHM value for the beam along the x- and y- directions, as well as the final cropped image, which can be used for diagnostic purposes
    return(xFWHM, yFWHM, finalimg)



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
Created on Fri Jan  26 01:00:00 2024

@author: leahghartman

Description : A file for functions that make images look prettier.
"""
# import random needed packages that should already be installed
import cv2
import numpy as np
import matplotlib.pyplot as plt
from PIL import Image
from scipy import ndimage

# import from other modules in the package
from gaussbean.utils import calc_utils

#########################
### START OF FUNCTIONS
#########################

def thru_median(mediansize, repeatamount=0, imgpath='', imgar=[]):
    """ Returns an image in the form of an array that has been run through a median filter a specified number of times.
    
        Parameters
        ----------
        mediansize : integer
            The size of the median filter in pixels (generally want this to be small; from 2-10 pixels).
        repeatamount (OPTIONAL) : integer
            The number of times the user wants to run the image through the median filter. Default is to run over the image only once.
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Array of the image if the user wants to input an array into the function rather than just an image path.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = calc_utils.check_array(imgpath, imgar)

    # create an array for the median filter images and append to this array when a change to the image is made
    result = [ndimage.median_filter(np.array(arrayimg), size=mediansize)]

    # for loop to repeat the number of times the median filter is run over the image
    for i in range(2, repeatamount+1):
        result.append(ndimage.median_filter(result[i-2], size=mediansize))

    # return the result after running through the median filter
    return(result[-1])

########################################################

def thru_lowpass(radius, imgpath='', imgar=[]):
    """ Returns an image in the form of an array that has been run through a low-pass filter one time.

        Parameters
        ----------
        radius : integer
            The radius of the mask used for the low-pass filter in pixels.
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        arimg (OPTIONAL) : array
            Array of the image if the user wants to input an array into the function rather than just an image path.
    """
    # set the array of the image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = calc_utils.check_array(imgpath, imgar)
    
    # perform the fourier transform and save the complex output
    ft = np.fft.fft2(arrayimg, axes=(0,1))

    # shift the origin to the center of the image
    ft_shift = np.fft.fftshift(ft)

    # create circular mask using size specified in the cell above
    mask = np.zeros_like(arrayimg) # --> returns an array of zeros the same shape as the image
    cy = mask.shape[0] // 2
    cx = mask.shape[1] // 2
    cv2.circle(mask, (cx,cy), radius, (255,255,255), -1)[0]

    # blur the mask to prevent artifacts on the image
    finalmask = cv2.GaussianBlur(mask, (19,19), 0)

    # apply the mask to the shifted fourier transform of the image
    masked_ft_shifted = np.multiply(ft_shift,finalmask) / 255

    # shift origin from center to upper left corner (basically return the image to its original state before using fft to get back)
    back_ishift_masked = np.fft.ifftshift(masked_ft_shifted)

    # do inverse fft and save as complex output
    filteredimg = np.fft.ifft2(back_ishift_masked, axes=(0,1))

    # combine complex real and imaginary components to form (the magnitude for) the original image again
    filteredimg = np.abs(filteredimg).clip(0,255).astype(np.uint8)

    # return the filtered image
    return(filteredimg)

########################################################

def back_subtract(origpath='', backpath='', origimgar=[], backimgar=[]):
    """ Returns an image in the form of an array after the background image provided is subtracted from the original image.

        Parameters
        ----------
        origpath (OPTIONAL) : string
            The path of the original image to be used (the image before background subtraction).
        backpath (OPTIONAL) : string
            The path of the background image to be subtracted from the original image (this should be a blank image).
        origimgar (OPTIONAL) : array
            Rather than an image path, one can opt to use an array instead. This is the array of the original image to be used (before background subtraction).
        backimgar (OPTIONAL) : array
            Rather than an image path, one can opt to use an array instead. This is the array of the background image to be used (a blank image).
    """
    # set the array of the original image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    origimg = calc_utils.check_array(origpath, origimgar)
    
    # set the array of the original image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    backimg = calc_utils.check_array(backpath, backimgar)

    # return the array of the image after background subtraction
    return(origimg - backimg)

########################################################

def crop_image(xpoint, ypoint, xmargins, ymargins, imgpath='', imgar=[]):
    """ Returns an image in the form of an array after being cropped the amount specified around the point specified.

        Parameters
        ----------
        xpoint : integer
            x-coordinate of the point at which the crop will be occurring.
        ypoint : integer
            y-coordinate of the point at which the crop will be occurring.
        xmargins : integer
            A number (in pixels) of how far in the x-direction, on either side of the cropping point, the user wants the image to be cropped.
        ymargins : integer
            A number (in pixels) of how far in the y-direction, on either side of the cropping point, the user wants the image to be cropped.
        imgpath (OPTIONAL) : string
            The path to the image that the user wants to run through the median filter.
        imgar (OPTIONAL) : array
            Array of the image if the user wants to input an array into the function rather than just an image path.
    """
    # set the array of the original image to whatever the user specifies (either based on the image path OR an array that the user inputs)
    arrayimg = calc_utils.check_array(imgpath, imgar)

    # crop the image
    finalimgar = arrayimg[round(ypoint-ymargins):round(ypoint+ymargins), round(xpoint-xmargins):round(xpoint+xmargins)]

    # return the cropped image array
    return(finalimgar)



from setuptools import setup, find_packages

VERSION = '1.0.2' 
DESCRIPTION = 'GAUSSian BEam ANalysis package'

# Setting up the package
setup(
        name="gaussbean", 
        version=VERSION,
        author="Leah Hartman",
        author_email="<leah.ghartman@gmail.com>",
        description=DESCRIPTION,
        packages=find_packages(),
        install_requires=[], # add any additional packages needed
        keywords=['python', 'gaussian', 'laser'],
        classifiers= []
)
